#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

# ##############
#   metadata   #
# ##############
__author__ = "Devin Bobadilla, Michelle van der Graaf"
#           YYYY.MM.DD
__date__ = "2020.02.14"
__version__ = (2, 7, 2)
__website__ = "https://github.com/Sigmmma/reclaimer"
__all__ = (
    "animation", "bitmaps", "h2", "h3", "halo_script", "hek", "meta", "misc",
    "model", "os_hek", "os_v3_hek", "os_v4_hek", "physics",
    "shadowrun_prototype", "sounds", "strings", "stubbs", "util",

    "common_descs", "constants", "data_extraction", "enums",
    "field_type_methods", "field_types",
    )
