#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...hek.defs.plac import *

#import and use the open saucified obje attrs
from .obje import *

# replace the object_type enum one that uses
# the correct default value for this object
obje_attrs = dict(obje_attrs)
obje_attrs[0] = dict(obje_attrs[0], DEFAULT=10)

plac_body = dict(plac_body)
plac_body[0] = obje_attrs

def get():
    return plac_def

plac_def = TagDef("plac",
    blam_header_stubbs('plac', 2),
    plac_body,

    ext=".placeholder", endian=">", tag_cls=ObjeTag
    )
